<!DOCTYPE html>
<html lang="en">
    <head>
        <title>Django testing site | {% block title %}{% endblock %}</title>
        {% block emailer_head %}{% endblock %}
    </head>
    <body>
        {% block content %}{% endblock %}
        {% block emailer_foot %}{% endblock %}
    </body>
</html>
