<!DOCTYPE html>

<html lang="en">

    <head>

        <title>
            Email Subscriptions
        </title>

    </head>

    <body>

        <p>
            You can select your email subscriptions using the link below:
        </p>
        <p>
            <a href="{{ subscriptions_url }}">Manage subscriptions</a>
        </p>
        <p>
            Please note that this link will only allow you to make changes once. To update your subscriptions later,
            request a new link or use the link at the bottom of any subscription email.
        </p>
        <p>
            If you did not request this email, no action is necessary.
        </p>

    </body>

</html>
