{% extends 'base.html' %}

{% load static %}

{% block title %}Email Subscriptions{% endblock %}

{% block emailer_head %}
    <link rel="stylesheet" type="text/css" href="{% static 'django_simple_bulk_emailer/css/django_simple_bulk_emailer.css' %}">
{% endblock %}

{% block content %}

    <div id="emailer_content">

        <h1>
            Email Subscriptions
        </h1>

        <p>
            Thank you. An email with instructions has been sent to the address provided.
        </p>

    </div>

{% endblock %}
